package com.j_spaces.example.customfifonotifications;

import net.jini.core.event.RemoteEvent;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;
import edu.emory.mathcs.backport.java.util.concurrent.ConcurrentSkipListMap;

public class BlockedOrderedQueue <T extends Comparable<T>> {
    // Maps the event sequence number to the event
    final private SizeConcurrentHashMap<T, RemoteEvent> _orderedEventsMap	=   new SizeConcurrentHashMap<T, RemoteEvent>();

    // Maps the event sequence number to the Thread that delivered the event
    final private ConcurrentSkipListMap _orderedThreadEventsMap	=
        new ConcurrentSkipListMap();


    final private long _maxCapacity;
    final private AtomicReference<T> _nextInOrder	= new AtomicReference<T>(null);
    final private AtomicReference<T> _maxInOrder	= new AtomicReference<T>(null);
    private final ComparableProvider<T> _provider;

    final private ThreadPark _fifodThreadPark = new ThreadPark();

    /**
     * Constructor
     * @param maxCapacity the maximum capacity of the queue
     * @param fifoThread the Thread that will read the events from the queue
     */
    public BlockedOrderedQueue(ComparableProvider<T> provider, long maxCapacity)
    {
        _maxCapacity = maxCapacity;
        _provider = provider;
    }

    /**
     * Add new event object to the queue.
     *
     * The operation will block if -
     * the queue exceeded its maximum capacity
     * and event sequence number exceeded
     *  the maximum sequence number in queue,
     *
     * @param theEvent
     */
    public void enqueue(RemoteEvent theEvent) throws InterruptedException
    {
        ThreadPark park = null;
        while( !tryInsert(theEvent))
        {

            if( park == null)
                park = new ThreadPark();

            // If insert failed - put the current thread into the thread map
            _orderedThreadEventsMap.put(getOrderValue(theEvent), park);

            // Another try
            if( tryInsert(theEvent))
            {
                _orderedThreadEventsMap.remove(getOrderValue(theEvent));
                break;
            }

            // Block current thread
            park.park();
        }
    }

    /**
     * Try to insert the event to the queue.
     * If the queue exceeded its maximum capacity
     * and the event sequence number exceeded
     * the maximum  sequence number in queue,
     * the insert will fail.
     * @param theEvent
     * @return true if event was inserted, false otherwise.
     */
    private boolean tryInsert( RemoteEvent theEvent)
    {
        if (_maxInOrder.get() == null) {
            _maxInOrder.set(_provider.getStartValue());
        }

        T max = _maxInOrder.get();
        T orderValue = getOrderValue(theEvent);

        if( orderValue.compareTo(max) < 0 || _orderedEventsMap.size() < _maxCapacity)
        {
            _maxInOrder.compareAndSet(max, getMax(max, orderValue));
            _orderedEventsMap.put( orderValue, theEvent);

            // Check if new event is the next event in queue
            if (orderValue.compareTo(_nextInOrder.get()) == 0)
            {
                _fifodThreadPark.unpark(); // release the FIFO thread
            }
            return true;
        }
        return false;
    }

    private T getMax(T value1, T value2) {
        int res = value1.compareTo(value2);
        return res >=0 ? value1 : value2;
    }

    /**
     * Remove and return the next event in queue.
     *
     * If the queue is empty
     * the method will block until an event arrives
     *
     * Note: This method is not concurrent and must be called from one thread only.
     *
     * @return the removed event
     * @throws InterruptedException
     */
    public RemoteEvent dequeue() throws InterruptedException
    {
        if (_nextInOrder.get() == null) {
            _nextInOrder.set(_provider.getStartValue());
        }
        
        // Remove the next event from the queue
        T next = _nextInOrder.get();
        RemoteEvent event = _orderedEventsMap.remove(next);

        // If queue is empty - block until some event arrives
        while( event == null)
        {
            // Block current thread
            _fifodThreadPark.park();
            if( Thread.currentThread().isInterrupted()) // only _fifoThread should get here
                throw new InterruptedException();
            event = _orderedEventsMap.remove(next);
        }

        boolean success = false;
        while (!success) {
            T currentValue = _nextInOrder.get();
            T nextValue = _provider.increment(currentValue);
            success = _nextInOrder.compareAndSet(currentValue, nextValue);
        }

        if( _maxCapacity < Long.MAX_VALUE) //no need to check for waiting threads
        {
            // Release the next Thread that is waiting on put
            Map.Entry entry =  _orderedThreadEventsMap.pollFirstEntry();
            if( entry != null)
            {
                ThreadPark park = (ThreadPark)entry.getValue();
                park.unpark();
            }
        }

        return event;
    }

    private T getOrderValue(RemoteEvent e) {
        return _provider.getOrderValue(e);
    }

    /**
     * Used to virtualize LockSuport, this class should be remove when LockSupport will be introduced be the 
     * Retrotranslator.
     *
     * @author	Guy Korland
     * @version	1.0
     * @since	5.2
     *
     * @see java.util.concurrent.locks.LockSupport
     */
    static private class ThreadPark
    {
        private boolean park = true;

        /**
         * Park the current thread, unless {@link #unpark()} was called before.
         * @throws InterruptedException
         */
        synchronized public void park() throws InterruptedException
        {
            if( park)
                wait();
            else
                park = true;
        }

        /**
         * Unark the parking thread.
         */
        synchronized public void unpark()
        {
            park = false;
            notifyAll();
        }

    }

}
