package com.j_spaces.example.customfifonotifications;

import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;

import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Comparator;



public class OrderingThread <T extends Comparable<T>> extends Thread {
    private static final Logger _logger = Logger.getLogger(com.gigaspaces.logger.Constants.LOGGER_CLIENT);

    private boolean shouldLive = true;

    // The events queue
    final transient private BlockedOrderedQueue _orderEventsMap;
    private final RemoteEventListener _listener;

    /**
     * starts a daemon order delegator thread
     *
     * @param threadName a name for identification purposes
     */
    public OrderingThread(RemoteEventListener listener, ComparableProvider<T> provider, int queueSize) {
        _orderEventsMap = new BlockedOrderedQueue<T>(provider, queueSize);
        _listener = listener;
        this.setName("ordering-thread");
        this.setDaemon(true);
        this.start();
    }

    /**
     * removes ordered events by order from the map, blocking if next in order
     * doesn't exist
     */
    public void run() {
        try {
            while (shouldLive) {
                RemoteEvent nextEvent = _orderEventsMap.dequeue();
                delegateEvent(nextEvent);
            }
        }
        catch (InterruptedException e) {
            if (_logger.isLoggable(Level.FINEST)) {
                _logger.log(Level.FINEST, this.getName() + " interrupted.", e);
            }
        }
    }

    /**
     * Delegate the event to user-supplied listener
     *
     * @param nextEvent the event to delegate
     */
    private void delegateEvent(RemoteEvent nextEvent) {
        try {
            _listener.notify(nextEvent); //delegate to user-supplied listener
        } catch (Throwable t) {
            if (_logger.isLoggable(Level.SEVERE)) {
                _logger.log(Level.SEVERE, "Notify failed while delegating to supplied listener on event: " + nextEvent , t);
            }
        }
    }

    /**
     * Notifies the listener of the ordered event that has just arrived.
     * Events may arrive out of order, thus sequenced before notification.
     *
     * @param theEvent the event triggered by a matching space operation.
     */
    public void notifyOrderEvent(RemoteEvent theEvent) {
        try {
            _orderEventsMap.enqueue(theEvent);
        }
        catch (InterruptedException e) {
            if (_logger.isLoggable(Level.FINEST)) {
                _logger.log(Level.FINEST, Thread.currentThread().getName() + " interrupted.", e);
            }
        }
    }

    /**
     * Graceful shutdown of OrderedThread, called upon invocation of
     * NotifyDelegator.close()
    */
    public void shutdown()
    {
        shouldLive = false;
    }
}
