/*
 * Copyright 2007 GigaSpaces Technologies Ltd. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS," WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. GIGASPACES WILL NOT 
 * BE LIABLE FOR ANY DAMAGE OR LOSS IN CONNECTION WITH THE SOFTWARE.
 */
/**
 * Title:        	GigaSpaces OMS Example - Client Application.
 * Description:    	Client application connects to the space and starts rounds of 
 * 					submitting one new order and taking multiple processed and rejected orders 
 * 					matching its client ID. 
 */

package com.gigaspaces.examples.tutorials.ordermanagement;

import com.j_spaces.core.IJSpace;					//	IJSpace class.
import com.j_spaces.core.client.FinderException; 	//	Finder exception.
import com.j_spaces.core.client.SpaceFinder;		//	Space finder.
import net.jini.core.entry.Entry; 					//	Entry Class.
import java.rmi.server.UID;							//	For creating a unique ID.

public class Client {
	
	static IJSpace space; 	// The space proxy.
	public Client(){} 		// Constructor.
	
	//	Pausing for number of ms
	public static void nap(long ms) {
		try {
			Thread.sleep(ms);
		} catch (InterruptedException iex) {
			System.err.println("Cannot sleep...");
		}
	}
	

	//		args -  Array of strings containing the space details. 
	//				The first cell (args[0]) contains the URL of the space that the client will connect to.
	//				this value is passed through the running script "bin\run_Client.bat".
	public static void main(String[] args) {
		System.out.println("\nWelcome to GigaSpaces OMS Example!\n");

		// Checks if any arguments were passed from batch file.
		if (args.length != 1) {
			// No argumants passed - the client will exit.
			System.out.println("Usage: <URL>");
			System.out.println("<protocol>://host:port/containername/spacename");
			System.exit(1);
		}
		
		try {
			// Find the Space using Gigaspaces SpaceFinder method.
			space = (IJSpace) SpaceFinder.find(args[0]); 
			if (space == null) {
				// Space not found - client exists.
				System.out.println("Space not found: " + args[0]);
				System.exit(-1);
			}
			//	Set space to fifo
			space.setFifo(true);
													
			// maxOrders holds the number of orders the client will submit.
			long maxOrders = 100000; 
						
			// 	Define arrays to hold processed orders and rejected orders taken from the space
			//	using mulitpletake operation.
			Entry processedOrders[] = null;
			Entry rejectedOrders[] = null;
			
			//	Used for creating unique ID for this client
			UID clientID = new UID();   
		    
			// Build a template used for taking only this client processed orders from the space.
			ProcessedOrder templateProcessedOrder = new ProcessedOrder();
			templateProcessedOrder.clientID = clientID.toString(); // Set the template to get only this client processed orders.
			
			// Build template for taking only this clients rejected orders from the space.
			Order templateRejectedOrder = new Order();
			templateRejectedOrder.clientID = clientID.toString(); // Set the template to get only this client orders.
			templateRejectedOrder.status = "Rejected"; // Set the template to get only rejected orders.
			
			// Reset Counters
			int countProcessedOrders = 0; // Counts number of processed orders taken from the space.
			int countRejectedOrders = 0; // Counts number of rejected orders taken from the space.
			
			//	Start rounds of placing one order and taking 
			// 	multiple processed orders and multiple rejected orders.
			// --------------------------------------------------------
			for (int i = 1; i < maxOrders + 1; i++) {
								
				// 	Prepare a new order and write it to the space.
				//	----------------------------------------------
				Order order = new Order();
				// Insert data to the order.
				order.data = new Integer(i);			// In this example the data is arbitrarily set equal to the order's number.
				order.clientID = clientID.toString();	// Stamp the order with this client unique ID.
				order.status = "New";					// Set as new order.
				order.number = new Integer(i);			// Set order number (to demonstrate fifo later).
				
				// Write the order to the space with 10min expiration time.
				space.write(order, null, 10 * 60000);
				// Display submitted order information.
				System.out.println("Order number " + i + " submitted by client ID[" + order.clientID + "]");
				
				// Take processed orders and rejected orders from the space 
				// --------------------------------------------------------
				// Try to take all the processed orders from the space.
				processedOrders = space.takeMultiple(templateProcessedOrder, null, Integer.MAX_VALUE);
				if (processedOrders != null) {
					countProcessedOrders = countProcessedOrders + (int)processedOrders.length; //	Update counter.
					System.out.println(processedOrders.length + " Processed orders found this round.");
				}
				else {
					System.out.println("0 processed orders found this round.");
				}
				// Try to take all the rejected orders from the space.
				rejectedOrders = space.takeMultiple(templateRejectedOrder, null, Integer.MAX_VALUE);
				if (rejectedOrders != null) {
					countRejectedOrders = countRejectedOrders + (int)rejectedOrders.length; //	Update Counter.
					System.out.println(rejectedOrders.length + " Rejected orders found this round.");
				}
				else {
					System.out.println("0 rejected orders found this round.");
				}
				
				//	Display this round's information.
				System.out.println("Status: " + i + " Orders submitted, " + countProcessedOrders +
									" Processed, " + countRejectedOrders + " Rejected");
				System.out.println("");
				
				//	Pausing for 2 sec to show results on console. 
				nap(2000);
			}
		} catch (FinderException ex) {
			System.out.println("Could not find space: " + args[0]);
			System.out.println("Please, start the GigaSpaces Server in order "+
				" to run this example.");
			

		} catch (Exception e) {
			if (e instanceof java.net.ConnectException) 
			{
				System.err.println(e.getMessage());
				System.exit(1);
			}
			
			if (e instanceof java.io.IOException) 
			{
				System.err.println(e.getMessage());
				System.exit(1);
			}
			
			System.err.println("transError problem..." + e.getMessage());
			e.printStackTrace();
		}

	}
	
	
}
