/*
 * Copyright 2007 GigaSpaces Technologies Ltd. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS," WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. GIGASPACES WILL NOT 
 * BE LIABLE FOR ANY DAMAGE OR LOSS IN CONNECTION WITH THE SOFTWARE.
 */
/**
 * Title:        	GigaSpaces OMS Example - Processor Application.
 * Description:   	The Processor application connects to the space and under transaction 
 * 					takes approved orders (order.status="Approved"), processes them 
 * 					and writes the processed orders (ProcessedOrder object) to the space. 
 */

package com.gigaspaces.examples.tutorials.ordermanagement;


import net.jini.core.transaction.server.TransactionManager;	//	Jini transaction manager.
import net.jini.core.transaction.Transaction;				//	Jini transaction.
import net.jini.core.transaction.TransactionFactory;		//	Jini transaction factory.

import com.j_spaces.core.IJSpace;							//	IJSpace.
import com.j_spaces.core.client.LocalTransactionManager;	//	Local transaction manager.
import com.j_spaces.core.client.FinderException;			//	Finder exception.
import com.j_spaces.core.client.SpaceFinder;				//	Space finder.
import java.net.InetAddress;								//	Used to get host address.




public class Processor {
	static String hostname = "";			//	The hostname that runs the validator.
	static TransactionManager txnManager; 	//	The local transaction manager object.
	static IJSpace space; 					//	The space object.
	Transaction tr; 						//	Transaction object.
	
	
	public static void main(String[] args) {
		//	Checks if any arguments were passed from batch file
		//	The first cell (args[0]) contains the URL of the space that the client will connect to.
		//	this value is passed through the running script "bin\run_Processor.bat".
		if (args.length != 1) {
			System.out.println("Usage: <spaceURL>");
			System.out.println("<protocol>://host:port/containername/spacename");
			System.exit(1);
		}
		
		try {
			hostname = InetAddress.getLocalHost().getHostAddress(); // Get the processor machine hostname.
			System.out.println("try to connect to " + args[0]);
			space = (IJSpace) SpaceFinder.find(args[0]);

			if (space == null) {
				System.out.println("Space not found: " + args[0]);
				System.exit(-1);
			}
			
			//	Set space to fifo.
			space.setFifo(true);
			
			System.out.println("connect to " + args[0] + " OK!" );
			System.out.println("Processor started at host " + hostname);
			
			//	Creating transaction object.
			Transaction tr = null;
			
			//	Create Order template to find approved orders (Order.status="Approved").
			Order template = new Order();
			template.status="Approved";
			
			Order order = new Order();	//	Holds the approved order taken. 
			ProcessedOrder processedOrder = new ProcessedOrder();	//	Holds the processed order to be written.
			
			//	Reset processed orders counter.
			int countProcessedOrders = 0;
	
			//	Start rounds of taking an approved order from the space, processing it and writing
			//	the processed order to the space.
			while (true) {
				try {
					// Start new transaction.
					tr = getTrans();
					//	Checking if transaction object ok.
					if (tr == null) {
						System.err.println("FATAL...  transaction is NULL");
						System.exit(1);
					}
					// Try to take an approved order from the space.
					System.out.println("Seeking approved orders...");
					order = (Order) space.take(template, tr, 30000);
					if (order != null) {
						System.out.println("Approved order number " + order.number + " found from client: " + order.clientID);
						// Create the processed order.
						processedOrder=order.process(hostname);
						// Write back the processed order.
						space.write(processedOrder, tr, 100 * 1000);
						// Commit transaction.
						tr.commit();
						//	Update the counter and display status.
						countProcessedOrders++;
						System.out.println("Order processed.");
						System.out.println("Processed " + countProcessedOrders + " orders.\n");
					}
					else  
						// No order found - aborting Transaction.
						tr.abort(); 
				} catch (Exception tx) {
					transError(tr, tx);
				}
			}
		} catch (FinderException ex) {
			System.out.println("Could not find space: " + args[0]);
			System.out.println("Please make sure that the GigaSpaces Server is started.");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	//	Method for handling transaction error exceptions.
	private static void transError(Transaction tr, Exception ex) {
		
		if (ex instanceof java.net.ConnectException) 
		{
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		
		if (ex instanceof java.io.IOException) 
		{
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		
		System.err.println("*****Processor Error: " + ex.getMessage());
		try {
			tr.abort();
		} catch (Exception ux) {
			System.err.println("transError problem..." + ux.getMessage());
		}
	}

	// Method for getting a transaction.
	private static Transaction getTrans() {
		Transaction.Created tcreate = null;
		try {
			if (txnManager == null) {
				txnManager = LocalTransactionManager.getInstance(space);
			}
			// txn timeout is 60 sec (60*1000ms)...
			tcreate = TransactionFactory.create(txnManager, 60 * 1000);
		} catch (net.jini.core.lease.LeaseDeniedException ldx) {
			System.err.println("Fatal... Lease denied.");
		} catch (java.rmi.RemoteException rx) {
			System.err.println("Fatal... Cannot create transaction manager.");
		}
		return tcreate.transaction;
	}
}
