/*
 * Copyright 2007 GigaSpaces Technologies Ltd. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS," WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY AND 
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. GIGASPACES WILL NOT 
 * BE LIABLE FOR ANY DAMAGE OR LOSS IN CONNECTION WITH THE SOFTWARE.
 */
/**
 * Title:        	GigaSpaces OMS Example - Validator Application.
 * Description:  - 	The Validator application connects to the space and under transaction 
 * 					takes new orders (order.status="New") stamps them as rejected or approved
 * 					(random validation) and writes them	back to the space. 
 */

package com.gigaspaces.examples.tutorials.ordermanagement;

import net.jini.core.transaction.server.TransactionManager;	//	Jini transaction manager.
import net.jini.core.transaction.Transaction;				//	Jini transaction.
import net.jini.core.transaction.TransactionFactory;		//	Jini transaction factory.

import com.j_spaces.core.IJSpace;							//	IJSpace.
import com.j_spaces.core.client.LocalTransactionManager;	//	Local transaction manager.
import com.j_spaces.core.client.FinderException;			//	Finder exception.
import com.j_spaces.core.client.SpaceFinder;				//	Space finder.
import java.net.InetAddress;	

public class Validator {
	static String hostname = "";			//	The hostname that runs the validator.
	static TransactionManager txnManager; 	//	The local transaction manager object.
	static IJSpace space; 					//	The space object.
	Transaction tr; 						//	Transaction object.
	
	public static void main(String[] args) {
		//	Checks if any arguments are passed from batch file
		//	The first cell (args[0]) contains the URL of the space that the client will connect to.
		//	this value is passed through the running script "bin\run_Validator.bat".
		if (args.length != 1) {
			System.out.println("Usage: <spaceURL>");
			System.out.println("<protocol>://host:port/containername/spacename");
			System.exit(1);
		}
		
		try {
			hostname = InetAddress.getLocalHost().getHostAddress(); // Get the validator machine hostname.
			
			// Connecting to the space.
			System.out.println("try to connect to " + args[0]);
			space = (IJSpace) SpaceFinder.find(args[0]);
			if (space == null) {
				System.out.println("Space not found: " + args[0]);
				System.exit(-1);
			}
		
			//	Set space to fifo
			space.setFifo(true);
			
			System.out.println("connect to " + args[0] + " OK!" );
			System.out.println("Validator started at host " + hostname);
			
			//	Creating transaction object
			Transaction tr = null;
			
			//	Create Order template to find new orders (Order.status="New").
			Order template = new Order();
			template.status="New";
			
			Order order = new Order();
			
			//	Reset validated orders counters
			int countApprovedOrders = 0;
			int countRejectedOrders = 0;
			
			String status; 
			
			//	Start rounds of taking a new order from the space, validating it and writing the
			//	order with the status changed back to the space.
			while (true) {
				try {
					// Start new transaction.
					tr = getTrans();
					//	Checking if transaction object is ok.
					if (tr == null) {
						System.err.println("FATAL...  transaction is NULL");
						System.exit(1);
					}
					// Try to take a new order from the space.
					System.out.println("Seeking new orders in the space...");
					order = (Order) space.take(template, tr, 30000);
					if (order != null) {
						System.out.println("New order number " + order.number + " found from client: " + order.clientID);
						// Validate the order.
						status=order.validate(hostname);
						// Write back the validated order.
						space.write(order, tr, 100 * 1000);
						// Commit transaction.
						tr.commit();
						//	Update the relevant counter and display the validation result.
						if (status=="Approved") {
							System.out.println("Order approved.");
							countApprovedOrders++; 
						}
						else 
						{
							System.out.println("Order rejected.");
							countRejectedOrders++;
						}
						System.out.println("Validator examined " + (countApprovedOrders + countRejectedOrders) +
										   " new orders: " + countApprovedOrders + " orders approved and " + 
										   countRejectedOrders + " orders rejected.\n");
					}
					else  
						// No order found - aborting Transaction.
						tr.abort(); 
				} catch (Exception tx) {
					transError(tr, tx);
				}
			}
		} catch (FinderException ex) {
			System.out.println("Could not find space: " + args[0]);
			System.out.println("Please make sure that the GigaSpaces Server is started.");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	//	Method for handling transaction error exceptions.
	private static void transError(Transaction tr, Exception ex) {
		
		if (ex instanceof java.net.ConnectException) 
		{
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		
		if (ex instanceof java.io.IOException) 
		{
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		
		System.err.println("*****Validator Error: " + ex.getMessage());
		try {
			tr.abort();
		} catch (Exception ux) {
			System.err.println("transError problem..." + ux.getMessage());
		}
	}
	
	//	Method for getting a transaction.
	private static Transaction getTrans() {
		Transaction.Created tcreate = null;
		try {
			if (txnManager == null) {
				txnManager = LocalTransactionManager.getInstance(space);
			}
			// txn timeout is 60 sec (60*1000ms)...
			tcreate = TransactionFactory.create(txnManager, 60 * 1000);
		} catch (net.jini.core.lease.LeaseDeniedException ldx) {
			System.err.println("Fatal... Lease denied.");
		} catch (java.rmi.RemoteException rx) {
			System.err.println("Fatal... Cannot create transaction manager.");
		}
		return tcreate.transaction;
	}
}
